/*
 *  Licensed to the Apache Software Foundation (ASF) under one
 *  or more contributor license agreements.  See the NOTICE file
 *  distributed with this work for additional information
 *  regarding copyright ownership.  The ASF licenses this file
 *  to you under the Apache License, Version 2.0 (the
 *  "License"); you may not use this file except in compliance
 *  with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an
 *   * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 *  KIND, either express or implied.  See the License for the
 *  specific language governing permissions and limitations
 *  under the License.
 */
package org.wso2.registry.app;

import org.apache.abdera.Abdera;
import org.apache.abdera.factory.Factory;
import org.apache.abdera.model.*;
import org.apache.abdera.parser.Parser;
import org.apache.abdera.protocol.server.Provider;
import org.apache.abdera.protocol.server.RequestContext;
import org.apache.abdera.protocol.server.ResponseContext;
import org.apache.abdera.protocol.server.impl.AbstractProvider;
import org.apache.abdera.protocol.server.impl.AbstractResponseContext;
import org.apache.abdera.protocol.server.impl.BaseResponseContext;
import org.apache.abdera.protocol.server.impl.EmptyResponseContext;
import org.apache.abdera.util.EntityTag;
import org.apache.abdera.util.MimeTypeHelper;
import org.apache.axiom.om.util.Base64;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.registry.Comment;
import org.wso2.registry.*;

import javax.activation.MimeType;
import javax.xml.namespace.QName;
import java.io.ByteArrayInputStream;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.Properties;

public class RegistryAtomProvider extends AbstractProvider
        implements Provider {
    private Log log = LogFactory.getLog(RegistryAtomProvider.class);

    private EntityTag service_etag = new EntityTag("registry");
    private Registry registry;


    public RegistryAtomProvider() {
        super(10);
        registry = (Registry) System.getProperties().get(RegistryConstants.REGISTRY);
//        try {
//            Resource r1 = new Resource();
//            r1.setDirectory(true);
//            r1.setAuthorUserName("Deepal");
//            registry.put("/wso2/wsas", r1);
//            registry.put("/wso2/esb", r1);
//            registry.put("/wso2/wsas/esb", r1);
//            Resource r2 = new Resource();
//            r2.setDirectory(false);
//            byte[] r1content = "R1 content".getBytes();
//            r2.setContent(r1content);
//            r2.setMediaType("text/xml");
//            r2.setAuthorUserName("Glen");
//            registry.put("/wso2/esb/r1", r2);
//        } catch (RegistryException e) {
//            e.printStackTrace();
//        }
    }

    private Document<Service> init_service_doc(Abdera abdera) {
        Factory factory = abdera.getFactory();
        Service service = factory.newService();
        Workspace workspace = service.addWorkspace("Registry");
        try {
            Collection collection = workspace.addCollection("registry", "atom/feed");
            collection.setAccept("entry");
            collection.addCategories().setFixed(false);
        } catch (Exception e) {
            log.error(e);
        }
        return service.getDocument();
    }


    private synchronized Document<Service> get_service_doc(Abdera abdera) {
        return init_service_doc(abdera);
    }

    private synchronized Document<Feed> get_feed_doc(Abdera abdera, String baseUri) {
        Factory factory = abdera.getFactory();
        Feed feed = factory.newFeed();
        try {
            feed.setId("http://wso2.org/registry,2007:feed");
            feed.setTitle("WSO2 Registry");
            feed.setUpdated(new Date());
            feed.addLink(baseUri);
            feed.addLink("", "self");
            return feed.getDocument();
        } catch (Exception e) {
            log.error(e);
        }
        return null;
    }

    public ResponseContext getService(
            RequestContext request) {
        Abdera abdera = request.getAbdera();
        Document<Service> service = get_service_doc(abdera);
        AbstractResponseContext rc;
        rc = new BaseResponseContext<Document<Service>>(service);
        rc.setEntityTag(service_etag);
        return rc;
    }

    /**
     * This method will check whether there are any request paramters , like ?version
     * if so need to act differently
     *
     * @param request : RequestContext
     * @return return true if there are any request parameters else false
     */
    private boolean isResourceOperation(RequestContext request) {
        String parameters[] = request.getParameterNames();
        if (parameters.length > 0) {
            if (RegistryConstants.PARAMETER_VERSION.equals(parameters[0]) ||
                    RegistryConstants.PARAMETER_TAGS.equals(parameters[0]) ||
                    RegistryConstants.PARAMETER_AVARAGE_REATINGS.equals(parameters[0]) ||
                    RegistryConstants.PARAMETER_REATINGS.equals(parameters[0]) ||
                    RegistryConstants.PARAMETER_COMMENTS.equals(parameters[0]) ||
                    RegistryConstants.PARAMETER_TAG_PATHS.equals(parameters[0])) {
                return true;
            }
        }
        return false;
    }

    /**
     * This method is used to get the resource path when ther is a parameter in the url
     *
     * @param request : RequestContext
     * @return : String value , after removing ? part
     */
    private String getResourcePath(RequestContext request) {
        String path = getEntryID(request);
        int index = path.indexOf("?");
        //index should always be >0
        path = path.substring(0, index);
        return path;
    }

    private String getArryAsString(String[] array) {
        String value = "";
        if (array != null) {
            for (int i = 0; i < array.length; i++) {
                String s = array[i];
                if (i < array.length - 1) {
                    value = value + s + ",";
                } else {
                    value = value + s;
                }
            }
        }
        return value;
    }

    public ResponseContext getFeed(
            RequestContext request) {
        Abdera abdera = request.getAbdera();
        AbstractResponseContext rc;
        if (isResourceOperation(request)) {
            String parameterName = request.getParameterNames()[0];
            if (RegistryConstants.PARAMETER_VERSION.equals(parameterName)) {
                String path = getResourcePath(request);
                try {
                    String[] versions = registry.getVersions(path);
                    Factory factory = abdera.getFactory();
                    Feed feed = factory.newFeed();
                    feed.setId("http://wso2.org/registry,2007:feed");
                    feed.setText("Resource Version for the resource" + path);
                    feed.setTitle("versions for " + path);
                    feed.setUpdated(new Date());
                    feed.addLink(request.getBaseUri() + "atom" + path);
                    feed.addLink("", "self");
                    feed.setText(getArryAsString(versions));
                    Document<Feed> docfeed = feed.getDocument();
                    rc = new BaseResponseContext<Document<Feed>>(docfeed);
                    rc.setEntityTag(calculateEntityTag(docfeed.getRoot()));
                    return rc;
                } catch (RegistryException e) {
                    return new EmptyResponseContext(500);
                }
                //Asking for the version numbers for the given resource
            } else if (RegistryConstants.PARAMETER_TAGS.equals(parameterName)) {
                String path = getResourcePath(request);
                Feed feed = getFeedElementsForTags(abdera, path);
                Document<Feed> docfeed = feed.getDocument();
                rc = new BaseResponseContext<Document<Feed>>(docfeed);
                rc.setEntityTag(calculateEntityTag(docfeed.getRoot()));
                return rc;
            } else if (RegistryConstants.PARAMETER_AVARAGE_REATINGS.equals(parameterName)) {
                String path = getResourcePath(request);
                Feed feed = getFeedElementsForAvarageRating(abdera, path);
                Document<Feed> docfeed = feed.getDocument();
                rc = new BaseResponseContext<Document<Feed>>(docfeed);
                rc.setEntityTag(calculateEntityTag(docfeed.getRoot()));
                return rc;
            } else if (RegistryConstants.PARAMETER_REATINGS.equals(parameterName)) {
                String path = getResourcePath(request);
                int lastIndex = path.lastIndexOf("/");
                String tagUser = path.substring(lastIndex + 1);
                path = path.substring(0, lastIndex);
                Feed feed = getFeedElementsForRating(abdera, path, tagUser);
                Document<Feed> docfeed = feed.getDocument();
                rc = new BaseResponseContext<Document<Feed>>(docfeed);
                rc.setEntityTag(calculateEntityTag(docfeed.getRoot()));
                return rc;
            } else if (RegistryConstants.PARAMETER_COMMENTS.equals(parameterName)) {
                String path = getResourcePath(request);
                Feed feed = getFeedElementsComments(abdera, path);
                Document<Feed> docfeed = feed.getDocument();
                rc = new BaseResponseContext<Document<Feed>>(docfeed);
                rc.setEntityTag(calculateEntityTag(docfeed.getRoot()));
                return rc;
            } else if (RegistryConstants.PARAMETER_TAG_PATHS.equals(parameterName)) {
                String tag = request.getParameter(RegistryConstants.PARAMETER_TAG_PATHS);
                Feed feed = getFeedElementsForTagPaths(abdera, tag);
                Document<Feed> docfeed = feed.getDocument();
                rc = new BaseResponseContext<Document<Feed>>(docfeed);
                rc.setEntityTag(calculateEntityTag(docfeed.getRoot()));
                return rc;
            } else {
                //We do not support
                return new EmptyResponseContext(500);
            }
        } else {
            String baseURI = request.getBaseUri().toString();
            String entry_id = getEntryID(request);
            Document<Feed> feed;
            if ("atom".equals(entry_id) || entry_id == null || "".equals(entry_id)) {
                entry_id = "/";
            }
            Feed feedObj = populateFeed(abdera, baseURI, entry_id);
            if (feedObj != null) {
                feed = feedObj.getDocument();
                rc = new BaseResponseContext<Document<Feed>>(feed);
                rc.setEntityTag(calculateEntityTag(feed.getRoot()));
                return rc;
            } else {
                //Server error
                EmptyResponseContext responseContext = new EmptyResponseContext(500);
                responseContext.setStatusText("Resource Not found");
                return responseContext;
            }
        }
    }

    private Feed getFeedElementsForTagPaths(Abdera abdera, String tag) {
        try {
            Factory factory = abdera.getFactory();
            Feed feed = factory.newFeed();
            TaggedResourcePath tagpaths[] = registry.getResourcePathsWithTag(tag);
            feed.setId("http://wso2.org/registry,2007:tags");
            feed.setTitle(tag + " [Tag Paths]");
            feed.setUpdated(new Date());
            feed.addLink("", "self");
            for (int i = 0; i < tagpaths.length; i++) {
                TaggedResourcePath tagPath = tagpaths[i];
                Entry entry = abdera.getFactory().newEntry();
                entry.setId("http://wso2.org/registry" + "atom" + tag);
                entry.setTitle(tagPath.getResourcePath());
                entry.setContent("" + tagPath.getTagCount());
                feed.addEntry(entry);
            }
            return feed;
        } catch (RegistryException e) {
            return null;
        }
    }

    private Feed getFeedElementsComments(Abdera abdera, String path) {
        try {
            Factory factory = abdera.getFactory();
            Feed feed = factory.newFeed();
            Comment comments[] = registry.getComments(path);
            feed.setId("http://wso2.org/registry,2007:tags");
            feed.setTitle(path + " [Comments]");
            feed.setUpdated(new Date());
            feed.addLink("", "self");
            for (int i = 0; i < comments.length; i++) {
                Comment comment = comments[i];
                Entry entry = abdera.getFactory().newEntry();
                entry.setId("http://wso2.org/registry" + "atom" + "comment");
                entry.setTitle("Resource Comment");
                entry.addAuthor(comment.getCommentedUser());
                entry.setUpdated(comment.getCommentedTime());
                entry.setContent(comment.getCommentText());
                feed.addEntry(entry);
            }
            return feed;
        } catch (RegistryException e) {
            return null;
        }
    }

    /**
     * This method is used to generate atom entry for a given resource
     *
     * @param abdera  : Abdera
     * @param baseURI String
     * @param path    : Resource Path
     * @return Generated Feed
     */
    private Feed populateFeed(Abdera abdera, String baseURI, String path) {
        try {
            Factory factory = abdera.getFactory();
            Feed feed = factory.newFeed();
            Resource resource = registry.get(path);
            return createFeedFromResource(resource, feed, path, baseURI, abdera , false);
        } catch (RegistryException e) {
            return null;
        }
    }

    private Feed createFeedFromResource(Resource resource,
                                        Feed feed,
                                        String path,
                                        String baseURI,
                                        Abdera abdera ,
                                        boolean executeQuery) throws RegistryException {
        Properties properties = resource.getProperties();
        if (properties != null) {
            Iterator keys = properties.keySet().iterator();
            while (keys.hasNext()) {
                String key = (String) keys.next();
                feed.setAttributeValue(key, (String) properties.get(key));
            }
        }
        if (resource.getState() == RegistryConstants.DELETED_STATE) {
            feed.setSubtitle("Deleted");
        }
        feed.setId("http://wso2.org/registry,2007:feed");
        feed.setText(resource.getDescription());
        feed.addAuthor(resource.getAuthorUserName());
        feed.setTitle(path);
        feed.setUpdated(new Date());
        feed.addLink(baseURI + "atom" + path);
        feed.addLink("", "self");

        if (resource.isDirectory() ||
                "comments".equals(resource.getMediaType())) {
            feed.setAttributeValue("directory", "true");
            Object o = resource.getContent();
            if (o instanceof String[]) {
                String values[] = (String[]) o;
                for (int i = 0; i < values.length; i++) {
                    String value = values[i];
                    Entry entry = abdera.getFactory().newEntry();
                    if(executeQuery){
                        entry.setTitle(value);
                    } else {
                        Resource childNode = registry.get(value);
                        entry.setId(baseURI + "atom" + childNode.getPath());
                        entry.setTitle(childNode.getPath());
                        entry.setSummary(resource.getDescription());
                        entry.setUpdated(childNode.getLastModified());
                        entry.setPublished(childNode.getCreatedTime());
                    }
                    entry.addLink(baseURI + "atom" + value);
                    feed.addEntry(entry);
                }
            } else {
                if (resource.isDirectory()) {
                    feed.setAttributeValue("directory", "true");
                } else {
                    feed.setAttributeValue("directory", "false");
                }
                Object content = resource.getContent();
                Entry entry = abdera.getFactory().newEntry();
                entry.setId(baseURI + "feed" + path);
                entry.setTitle(path);
                entry.setSummaryAsHtml(resource.getDescription());
                entry.setUpdated(resource.getLastModified());
                entry.setPublished(resource.getCreatedTime());
                if (content != null && content instanceof byte[]) {
                    ByteArrayInputStream in = new ByteArrayInputStream((byte[]) content);
                    entry.setContent(in);
                } else {
                    if (content != null) {
                        entry.setContent(content.toString());
                    }
                }
                feed.addEntry(entry);
            }
        } else {
            feed.setAttributeValue("directory", "false");
            Object content = resource.getContent();
            Entry entry = abdera.getFactory().newEntry();
            entry.setId(baseURI + "feed" + path);
            entry.setTitle(path);
            entry.setSummaryAsHtml(resource.getDescription());
            entry.setUpdated(resource.getLastModified());
            entry.setPublished(resource.getCreatedTime());
            if (content != null && content instanceof byte[]) {
                ByteArrayInputStream in = new ByteArrayInputStream((byte[]) content);
                entry.setContent(in);
            } else {
                if (content != null) {
                    entry.setContent(content.toString());
                }
            }
            feed.addEntry(entry);
        }
        return feed;
    }

    private Feed getFeedElementsForTags(Abdera abdera, String path) {
        try {
            Factory factory = abdera.getFactory();
            Feed feed = factory.newFeed();
            Tag tags[] = registry.getTags(path);

            feed.setId("http://wso2.org/registry,2007:tags");
            feed.setTitle(path + " [Tags]");
            feed.setUpdated(new Date());
            feed.addLink("", "self");

            for (int i = 0; i < tags.length; i++) {
                Tag tag = tags[i];
                Entry entry = abdera.getFactory().newEntry();
                entry.setId("http://wso2.org/registry" + "atom" + tag);
                entry.setTitle(tag.getTagName());
                entry.setSummary("" + tag.getTagCount());
                feed.addEntry(entry);
            }
            return feed;
        } catch (RegistryException e) {
            return null;
        }
    }

    private Feed getFeedElementsForAvarageRating(Abdera abdera, String path) {
        try {
            Factory factory = abdera.getFactory();
            Feed feed = factory.newFeed();
            feed.setId("http://wso2.org/registry,2007:tags");
            feed.setTitle(path + " [Ratings]");
            feed.setUpdated(new Date());
            feed.addLink("", "self");
            feed.setSubtitle("" + registry.getAverageRating(path));
            return feed;
        } catch (RegistryException e) {
            return null;
        }
    }

    private Feed getFeedElementsForRating(Abdera abdera, String path, String userName) {
        try {
            Factory factory = abdera.getFactory();
            Feed feed = factory.newFeed();
            feed.setId("http://wso2.org/registry,2007:tags");
            feed.setTitle(path + " [Ratings]");
            feed.setUpdated(new Date());
            feed.addLink("", "self");
            feed.setSubtitle("" + registry.getRating(path, userName));
            return feed;
        } catch (RegistryException e) {
            return null;
        }
    }


    @SuppressWarnings("unchecked")
    public ResponseContext createEntry(
            RequestContext request) {
        Abdera abdera = request.getAbdera();
        Parser parser = abdera.getParser();
        try {
            MimeType contentType = request.getContentType();
            String ctype = (contentType != null) ? contentType.toString() : null;
            if (ctype != null && !MimeTypeHelper.isAtom(ctype) && !MimeTypeHelper.isXml(ctype))
                return new EmptyResponseContext(415);

            Document<Entry> entry_doc =
                    (Document<Entry>) request.getDocument(parser).clone();
            if (entry_doc != null) {
                Entry entry = entry_doc.getRoot();
                Resource resource = new Resource();
                String attValue = entry.getAttributeValue("directory");
                List attributes = entry.getAttributes();
                for (int i = 0; i < attributes.size(); i++) {
                    QName attName = (QName) attributes.get(i);
                    String key = attName.getLocalPart();
                    if (!("directory".equals(key) || "mediaType".equals(key))) {
                        resource.setProperty(key, entry.getAttributeValue(key));
                    }
                }
                String mediaType = entry.getAttributeValue("mediaType");
                if (mediaType != null) {
                    resource.setMediaType(mediaType);
                }
                resource.setAuthorUserName(entry.getAuthor().getName());
                resource.setDescription(entry.getSummary());
                if (attValue != null && "true".equals(attValue)) {
                    // Directoy
                    resource.setDirectory(true);
                } else {
                    // Resource
                    String content1 = entry.getContent();
                    if (content1 != null && !Content.Type.TEXT.equals(entry.getContentType())) {
                        byte[] content = Base64.decode(content1);
                        resource.setContent(content);
                    } else {
                        resource.setContent(content1);
                    }
                }
                registry.put(entry.getTitle(), resource);
            }
            return new EmptyResponseContext(200);
        } catch (Exception e) {
            return new EmptyResponseContext(500);
        }
    }

    public ResponseContext deleteEntry(
            RequestContext request) {
        String entry_id = getEntryPath(request);
        if (!"".equals(entry_id)) {
            try {
                registry.delete(entry_id);
                return new EmptyResponseContext(204);
            } catch (RegistryException e) {
                return new EmptyResponseContext(500);
            }
        } else {
            return new EmptyResponseContext(500);
        }
    }

    public ResponseContext getEntry(
            RequestContext request) {
        Abdera abdera = request.getAbdera();
        String baseURI = request.getBaseUri().toString();
        String entry_id = getEntryPath(request);
        Entry entry = null;
        if ("".equals(entry_id)) {
            try {
                Resource reosurce = registry.get(RegistryConstants.PATH_SEPARATOR);
                if (reosurce == null) {
                    entry = getEmptyEntry(abdera, baseURI, get_feed_doc(abdera, baseURI));
                }
            } catch (RegistryException e) {
                log.error(e);
                return new EmptyResponseContext(404);
            }
        } else {

        }
        if (entry != null) {
            Feed feed = entry.getParentElement();
            entry.setSource(feed.getAsSource());
            Document<Entry> entry_doc = entry.getDocument();
            AbstractResponseContext rc = new BaseResponseContext<Document<Entry>>(entry_doc);
            rc.setEntityTag(calculateEntityTag(entry));
            return rc;
        } else {
            return new EmptyResponseContext(404);
        }
    }

    @SuppressWarnings("unchecked")
    public ResponseContext updateEntry(
            RequestContext request) {
        Abdera abdera = request.getAbdera();
        Parser parser = abdera.getParser();
        try {
            MimeType contentType = request.getContentType();
            if (contentType != null && !MimeTypeHelper.isAtom(contentType.toString()))
                return new EmptyResponseContext(415);
            Document<Entry> entry_doc =
                    (Document<Entry>) request.getDocument(parser).clone();
            if (entry_doc != null) {
                Entry entry = entry_doc.getRoot();
                String title = entry.getSummary();
                if (RegistryConstants.PARAMETER_RESTORE.equals(title)) {
                    String version = entry.getContent();
                    registry.restoreVersion(version);
                } else if (RegistryConstants.PARAMETER_TAGS.equals(title)) {
                    String path = entry.getTitle();
                    String tags = entry.getContent();
                    registry.applyTag(path, tags);
                } else if (RegistryConstants.PARAMETER_TAGS_REMOVED.equals(title)) {
                    String path = entry.getTitle();
                    String tag = entry.getContent();
                    registry.removeTag(path, tag);
                } else if (RegistryConstants.PARAMETER_REATINGS.equals(title)) {
                    String path = entry.getTitle();
                    String rate = entry.getContent();
                    registry.rateResource(path, Integer.parseInt(rate));
                } else if (RegistryConstants.PARAMETER_COMMENTS.equals(title)) {
                    String path = entry.getTitle();
                    String author = entry.getAuthor().getName();
                    String commentString = entry.getContent();
                    Comment comment = new Comment();
                    comment.setCommentedTime(entry.getUpdated());
                    comment.setCommentedUser(author);
                    comment.setCommentText(commentString);
                    registry.addComment(path, comment);
                } else if (RegistryConstants.PARAMETER_LOGS.equals(title)) {
                    Date to = entry.getEdited();
                    Date from = entry.getUpdated();
                    String action = entry.getTitle();
                    String author = entry.getAuthor().getName();
                    String path = entry.getContent();
                    Feed feed = getFeedElementsForLogs(abdera, path, from, to, author, action);
                    Document<Feed> docfeed = feed.getDocument();
                    AbstractResponseContext rc;
                    rc = new BaseResponseContext<Document<Feed>>(docfeed);
                    rc.setEntityTag(calculateEntityTag(docfeed.getRoot()));
                    return rc;
                } else if (RegistryConstants.PARAMETER_QUERY.equals(title)) {
                    String path = entry.getTitle();
                    String[] parameters = RemoteRegistry.decodeParametesAsString(entry.getContent());
                    Resource resource = registry.executeQuery(path, parameters);

                    Factory factory = abdera.getFactory();
                    Feed feed = factory.newFeed();
                    feed = createFeedFromResource(resource,
                            feed,
                            path,
                            request.getBaseUri().toString(),
                            abdera , true);
                    Document<Feed> docfeed = feed.getDocument();
                    AbstractResponseContext rc;
                    rc = new BaseResponseContext<Document<Feed>>(docfeed);
                    rc.setEntityTag(calculateEntityTag(docfeed.getRoot()));
                    return rc;
                }

            }
            return new EmptyResponseContext(200);
        } catch (Exception pe) {
            return new EmptyResponseContext(500);
        }
    }

    private Feed getFeedElementsForLogs(Abdera abdera,
                                        String path,
                                        Date from,
                                        Date to,
                                        String author,
                                        String action) throws RegistryException {
        LogEntry[] entries = registry.getLogs(
                path, Integer.parseInt(action), author, from, to);
        Factory factory = abdera.getFactory();
        Feed feed = factory.newFeed();
        feed.setId("http://wso2.org/registry,2007:tags");
        feed.setTitle(path + " [Logs]");
        feed.setUpdated(new Date());
        feed.addLink("", "self");
        for (int i = 0; i < entries.length; i++) {
            LogEntry logentry = entries[i];
            Entry entry = abdera.getFactory().newEntry();
            entry.setEdited(logentry.getDate());
            entry.setContent(logentry.getActionData());
            entry.addAuthor(logentry.getUserName());
            entry.setSummary("" + logentry.getAction());
            entry.setTitle(logentry.getResourcePath());
            feed.addEntry(entry);
        }
        return feed;
    }

    private EntityTag calculateEntityTag(Base base) {
        String id = null;
        String modified = null;
        if (base instanceof Entry) {
            id = ((Entry) base).getId().toString();
            modified = ((Entry) base).getUpdatedElement().getText();
        } else if (base instanceof Feed) {
            id = ((Feed) base).getId().toString();
            modified = ((Feed) base).getUpdatedElement().getText();
        }
        return EntityTag.generate(id, modified);
    }

    private Entry getEmptyEntry(Abdera abdera, String baseURI, Document<Feed> feed) {
        Entry entry = abdera.getFactory().newEntry();
        entry.setId(baseURI + "atom/");
        entry.setTitle("Registry root");
        entry.setSummaryAsHtml("No Resources found in the Registry");
        entry.setUpdated(new Date());
        entry.setPublished(new Date());
        entry.addLink(baseURI + "atom/");
        feed.getRoot().addEntry(entry);
        return entry;
    }

    private String getEntryID(RequestContext request) {
        String path = request.getBaseUri().getPath();
        String uri = request.getUri().toString();
        return uri.replaceAll(path + "atom", "");
    }

    private String getEntryPath(RequestContext request) {
        String path = request.getBaseUri().getPath();
        String uri = request.getUri().toString();
        return uri.replaceAll(path + "feed", "");
    }

}
