/*
 * Copyright (c) 2006, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.web;

import org.wso2.registry.servlet.Utils;
import org.wso2.registry.servlet.FileUploadUtil;
import org.wso2.registry.*;
import org.wso2.registry.secure.SecureRegistry;
import org.wso2.registry.web.UIConstants;
import org.wso2.registry.web.actions.*;

import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.ServletException;
import javax.servlet.ServletConfig;
import java.io.IOException;

public class ControllerServlet extends HttpServlet {

    protected transient ServletConfig servletConfig;

    //To store the conetxt path
    private String contextRoot = null;

    protected void doPost(HttpServletRequest request,
                          HttpServletResponse response)
            throws ServletException, IOException {
        initContextRoot(request);

        // if a path is not set in the session, set the path to the root
        String path = (String) request.getSession().getAttribute(UIConstants.PATH_ATTR);
        if (path == null || path.length() == 0) {
            path = RegistryConstants.ROOT_PATH;
            request.getSession().setAttribute(UIConstants.PATH_ATTR, RegistryConstants.ROOT_PATH);
        }

        // set the user name for the session
        SecureRegistry userRegistry =
                (SecureRegistry) request.getSession().getAttribute(ConsoleConstants.USER_REGISTRY);
        if (userRegistry != null) {
            request.getSession().setAttribute(UIConstants.USER_ATTR, userRegistry.getUserID());
        } else {
            request.getSession().setAttribute(UIConstants.USER_ATTR, RegistryConstants.ANONYMOUS_USER);
        }

        String uri = request.getRequestURI();
        String controlPart = uri.substring(contextRoot.length(), uri.length());

        if (controlPart.startsWith(RegistryConstants.PATH_SEPARATOR + UIConstants.SYSTEM_PATH)) {

            // process system requests

            String command = uri.substring((contextRoot + RegistryConstants.PATH_SEPARATOR + UIConstants.SYSTEM_PATH).
                    length(), uri.length());

            if (command.equals("/signin")) {

                SignInAction signInAction = new SignInAction();
                signInAction.setUserName(request.getParameter("userName"));
                signInAction.setPassword(request.getParameter("password"));

                try {
                    signInAction.execute(request);
                } catch (Exception e) {
                    setErrorMessage(request, e.getMessage());
                    e.printStackTrace();
                }

                response.sendRedirect("/wso2registry/web" + path);

                //forwardToResources(request, response, path);

            } else if (command.equals("/signout")) {

                SignInAction signInAction = new SignInAction();
                signInAction.setUserName(RegistryConstants.ANONYMOUS_USER);
                signInAction.setPassword(request.getParameter(RegistryConstants.ANONYMOUS_PASSWORD));

                try {
                    signInAction.execute(request);
                } catch (Exception e) {
                    setErrorMessage(request, e.getMessage());
                    e.printStackTrace();
                }

                response.sendRedirect("/wso2registry/web" + path);
                //forwardToResources(request, response, path);

            } else if (command.equals("/addUser")) {

                AddUserAction addUserAction = new AddUserAction();
                addUserAction.setNewUserName(request.getParameter("newUserName"));
                addUserAction.setPassword(request.getParameter("password"));
                addUserAction.setConfirmedPassword(request.getParameter("confirmedPassword"));
                addUserAction.setRoleName(request.getParameter("roleName"));

                try {
                    addUserAction.execute(request);
                } catch (Exception e) {
                    setErrorMessage(request, e.getMessage());
                    e.printStackTrace();
                }

                response.sendRedirect("/wso2registry/system/people");
                //forwardToUserManagement(request, response);

            } else if (command.equals("/addRole")) {

                AddRoleAction addRoleAction = new AddRoleAction();
                addRoleAction.setNewRoleName(request.getParameter("newRoleName"));
                try {
                    addRoleAction.execute(request);
                } catch (Exception e) {
                    setErrorMessage(request, e.getMessage());
                    e.printStackTrace();
                }

                response.sendRedirect("/wso2registry/system/people");
                //forwardToUserManagement(request, response);

            } else if (command.equals("/tag")) {

                AddTagAction addTagAction = new AddTagAction();
                addTagAction.setPath(path);
                addTagAction.setTag(request.getParameter("tag"));

                try {
                    addTagAction.execute(request);
                } catch (Exception e) {
                    setErrorMessage(request, e.getMessage());
                    e.printStackTrace();
                }

                response.sendRedirect("/wso2registry/web" + path);
                //forwardToResources(request, response, path);

            } else if (command.equals("/authorize")) {

                AuthorizationAction authorizationAction = new AuthorizationAction();
                authorizationAction.setPathToAuthorize(path);
                authorizationAction.setUserToAuthorize(request.getParameter("userToAuthorize"));
                authorizationAction.setActionToAuthorize(request.getParameter("actionToAuthorize"));
                authorizationAction.setPermissionType(request.getParameter("permissionType"));

                try {
                    authorizationAction.execute(request);
                } catch (Exception e) {
                    setErrorMessage(request, e.getMessage());
                    e.printStackTrace();
                }

                response.sendRedirect("/wso2registry/web" + path);
                //forwardToResources(request, response, path);

            } else if (command.equals("/authorize/mass")) {

                MassAuthorizationAction massAuthorizationAction = new MassAuthorizationAction();
                massAuthorizationAction.setResourcePath(path);
                massAuthorizationAction.setPermissionString(request.getParameter("permissionInput"));

                try {
                    massAuthorizationAction.execute(request);
                } catch (RegistryException e) {
                    setErrorMessage(request, e.getMessage());
                    e.printStackTrace();
                }

                response.sendRedirect("/wso2registry/web" + path);
                //forwardToResources(request, response, path);

            } else if (command.equals("/authorizeRole")) {

                RoleAuthorizationAction roleAuthorizationAction = new RoleAuthorizationAction();
                roleAuthorizationAction.setPathToAuthorize(path);
                roleAuthorizationAction.setRoleToAuthorize(request.getParameter("roleToAuthorize"));
                roleAuthorizationAction.setActionToAuthorize(request.getParameter("actionToAuthorize"));
                roleAuthorizationAction.setPermissionType(request.getParameter("permissionType"));

                try {
                    roleAuthorizationAction.execute(request);
                } catch (Exception e) {
                    setErrorMessage(request, e.getMessage());
                    e.printStackTrace();
                }

                response.sendRedirect("/wso2registry/web" + path);
                //forwardToResources(request, response, path);

            } else if (command.equals("/authorizeRoles")) {

                MassRoleAuthorizationAction massRoleAuthorizationAction = new MassRoleAuthorizationAction();
                massRoleAuthorizationAction.setResourcePath(path);
                massRoleAuthorizationAction.setPermissionString(request.getParameter("permissionInput"));

                try {
                    massRoleAuthorizationAction.execute(request);
                } catch (RegistryException e) {
                    setErrorMessage(request, e.getMessage());
                    e.printStackTrace();
                }

                response.sendRedirect("/wso2registry/web" + path);
                //forwardToResources(request, response, path);

            } else if (command.equals("/setDescription")) {

                String resourcePath = path;
                String description = request.getParameter("description");

                SetDescriptionAction setDescriptionAction = new SetDescriptionAction();
                setDescriptionAction.setResourcePath(resourcePath);
                setDescriptionAction.setDescription(description);

                try {
                    setDescriptionAction.execute(request);
                } catch (RegistryException e) {
                    setErrorMessage(request, e.getMessage());
                    e.printStackTrace();
                }

                request.getSession().setAttribute(UIConstants.AJAX_DESCRIPTION_STRING, setDescriptionAction.getDescription());
                request.getRequestDispatcher(UIConstants.AJAX_DESCRIPTION_JSP).forward(request, response);

            } else if (command.equals("/setRating")) {

                int userRating = new Integer(request.getParameter("ratingValue")).intValue();

                AjaxRatingAction ajaxRatingAction = new AjaxRatingAction();
                ajaxRatingAction.setResourcePath(path);
                ajaxRatingAction.setUserRating(userRating);

                try {
                    ajaxRatingAction.execute(request);
                } catch (RegistryException e) {
                    setErrorMessage(request, e.getMessage());
                    e.printStackTrace();
                }

                request.getSession().setAttribute(UIConstants.AJAX_RATING_BEAN, ajaxRatingAction);
                request.getRequestDispatcher(UIConstants.AJAX_RATING_JSP).forward(request, response);

            } else if (command.equals("/addComment")) {

                CommentAction commentAction = new CommentAction();
                commentAction.setPath(path);
                commentAction.setCommentText(request.getParameter("comment"));

                try {
                    commentAction.execute(request);
                } catch (Exception e) {
                    setErrorMessage(request, e.getMessage());
                    e.printStackTrace();
                }

                response.sendRedirect("/wso2registry/web" + path);
                //forwardToResources(request, response, path);

            } else if (command.equals("/addUserToRole")) {

                String userName = (String) request.getParameter("userName");
                AddUserRoleAction addUserRoleAction = new AddUserRoleAction();
                addUserRoleAction.setUserName(userName);
                addUserRoleAction.setRoleName(request.getParameter("roleToAdd"));

                try {
                    addUserRoleAction.execute(request);
                } catch (Exception e) {
                    setErrorMessage(request, e.getMessage());
                    e.printStackTrace();
                }

                response.sendRedirect("/wso2registry/system/people/" + userName);
                //forwardToUserPage(request, response, request.getParameter("userName"));

            } else if (command.equals("/addCollection")) {

                AddCollectionAction addCollectionAction = new AddCollectionAction();
                addCollectionAction.setParentPath(request.getParameter("parentPath"));
                addCollectionAction.setCollectionName(request.getParameter("collectionName"));
                addCollectionAction.setMediaType(request.getParameter("mediaType"));
                addCollectionAction.setDescription(request.getParameter("description"));

                try {
                    addCollectionAction.execute(request);
                } catch (RegistryException e) {
                    setErrorMessage(request, e.getMessage());
                    e.printStackTrace();
                }

                response.sendRedirect("/wso2registry/web" + path);
                //forwardToResources(request, response, path);

            } else if (command.equals("/addResource")) {

                handleFileUpload(request, response);
            }
        }
    }

    private void handleFileUpload(HttpServletRequest request, HttpServletResponse response)
            throws IOException, ServletException {
        try {
            Registry registry = Utils.getSecureRegistry(request);

            Resource fileElement =
                    FileUploadUtil.processUpload(request);
            String path = fileElement.getPath();

            registry.put(path, fileElement);

            response.setContentType("text/html");

            String parentPath = getParentPath(path);
            ResourceDetailsAction details = new ResourceDetailsAction();
            details.setPath(parentPath);
            details.execute(request);

            CollectionViewAction collection = null;
            if (details.isCollection()) {
                collection = new CollectionViewAction();
                collection.setPath(parentPath);
                collection.execute(request);
            }

            request.getSession().setAttribute(UIConstants.RESOURCE_BEAN, details);
            request.getSession().setAttribute(UIConstants.COLLECTION_BEAN, collection);

            request.getRequestDispatcher(UIConstants.RESOURCES_JSP)
                    .forward(request, response);

        } catch (Exception e) {

            e.printStackTrace();
            response.setContentType("text/html");
            request.getRequestDispatcher(UIConstants.RESOURCES_JSP)
                    .forward(request, response);
        }
    }

    protected void doGet(HttpServletRequest request,
                         HttpServletResponse response)
            throws ServletException, IOException {

        // if a path is not set in the session, set the path to the root
        String path = (String) request.getSession().getAttribute(UIConstants.PATH_ATTR);
        if (path == null || path.length() == 0) {
            path = RegistryConstants.ROOT_PATH;
            request.getSession().setAttribute(UIConstants.PATH_ATTR, RegistryConstants.ROOT_PATH);
        }

        // set the user name for the session
        SecureRegistry userRegistry =
                (SecureRegistry) request.getSession().getAttribute(ConsoleConstants.USER_REGISTRY);
        if (userRegistry != null) {
            request.getSession().setAttribute(UIConstants.USER_ATTR, userRegistry.getUserID());
        } else {
            request.getSession().setAttribute(UIConstants.USER_ATTR, RegistryConstants.ANONYMOUS_USER);
        }

        initContextRoot(request);

        String uri = request.getRequestURI();

        String controlPart = uri.substring(contextRoot.length());
        if (controlPart.startsWith(RegistryConstants.PATH_SEPARATOR + UIConstants.WEB_PATH)) {

            // process request regarding resource handling

            if (uri.equals("") || uri.endsWith(UIConstants.WEB_PATH + RegistryConstants.PATH_SEPARATOR)) {
                path = RegistryConstants.ROOT_PATH;

            } else {
                path = uri.substring((contextRoot + RegistryConstants.PATH_SEPARATOR + UIConstants.WEB_PATH).
                        length(), uri.length());
            }

            // if user is browsing an old version of the resource, we append it to the path, so that
            // the backend registry gives the details of the version
            String qPart = request.getQueryString();
            if (qPart != null) {
                if (qPart.startsWith("v")) {

                    // check if URL is for the resource content (i.e. /mycollection/myreosurce?v2&content

                    String[] q = qPart.split("&");
                    if (q.length == 2) {
                        if (q[1].equals("content")) {
                            path = path + "?" + q[0];
                            sendResourceContent(request, response, path);
                            return;
                        }
                    } else {
                        path = path + "?" + qPart;
                    }

                } else if (qPart.equals("content")) {
                    sendResourceContent(request, response, path);
                    return;
                }
            }

            // whenever a user selects a URL for a resource path, we set that path in the user's
            // session. so the user's path is known from any part of the UI.
            request.getSession().setAttribute(UIConstants.PATH_ATTR, path);

            forwardToResources(request, response, path);

        }  else if (controlPart.startsWith(RegistryConstants.PATH_SEPARATOR + UIConstants.SYSTEM_PATH)) {

            // process system requests

            String command = uri.substring((contextRoot + RegistryConstants.PATH_SEPARATOR + UIConstants.SYSTEM_PATH).
                    length(), uri.length());

            if (command.equals("/people")) {
                forwardToUserManagement(request, response);

            } else if (command.equals("/signout")) {

                SignInAction signInAction = new SignInAction();
                signInAction.setUserName(RegistryConstants.ANONYMOUS_USER);
                signInAction.setPassword(RegistryConstants.ANONYMOUS_PASSWORD);

                try {
                    signInAction.execute(request);
                } catch (Exception e) {
                    setErrorMessage(request, e.getMessage());
                    e.printStackTrace();
                }

                response.sendRedirect("/wso2registry/web" + path);
                //forwardToResources(request, response, path);

            } else if (command.equals("/activity")) {

                forwardToActivity(request, response);

            } else if (command.equals("/activity/new")) {

                forwardToEmptyActivity(request, response);

            } else if (command.equals("/search/advanced")) {

                forwardToAdvancedSearch(request, response);

            } else if (command.equals("/search/advanced/new")) {

                forwardToNewAdvancedSearch(request, response);

            } else if (command.startsWith("/people") && command.length() > "/people/.".length()) {

                String userName = uri.substring("/wso2registry/system/people/".length(), uri.length());

                forwardToUserPage(request, response, userName);

            } else if (command.equals("/removeUserFromRole")) {

                RemoveUserRoleAction removeUserRoleAction = new RemoveUserRoleAction();
                removeUserRoleAction.setUserName(request.getParameter("user"));
                removeUserRoleAction.setRoleName(request.getParameter("role"));

                try {
                    removeUserRoleAction.execute(request);
                } catch (Exception e) {
                    setErrorMessage(request, e.getMessage());
                    e.printStackTrace();
                }

                response.sendRedirect("/wso2registry/system/people/" + request.getParameter("user"));
                //forwardToUserPage(request, response, request.getParameter("user"));

            } else if (command.equals("/removeUser")) {

                RemoveUserAction removeUserAction = new RemoveUserAction();
                removeUserAction.setUserName(request.getParameter("user"));

                try {
                    removeUserAction.execute(request);
                } catch (Exception e) {
                    setErrorMessage(request, e.getMessage());
                    e.printStackTrace();
                }

                response.sendRedirect("/wso2registry/system/people");
                //forwardToUserManagement(request, response);

            } else if (command.equals("/removeRole")) {

                RemoveRoleAction removeRoleAction = new RemoveRoleAction();
                removeRoleAction.setRoleName(request.getParameter("role"));

                try {
                    removeRoleAction.execute(request);
                } catch (Exception e) {
                    setErrorMessage(request, e.getMessage());
                    e.printStackTrace();
                }

                response.sendRedirect("/wso2registry/system/people");
                //forwardToUserManagement(request, response);

            } else if (command.equals("/search")) {

                forwardToSearch(request, response);

            } else if (command.equals("/getInitialRating")) {

                AjaxRatingAction ajaxRatingAction = new AjaxRatingAction();
                ajaxRatingAction.setResourcePath(request.getParameter("resourcePath"));
                try {
                    ajaxRatingAction.execute(request);
                } catch (RegistryException e) {
                    e.printStackTrace();
                }

                request.getSession().setAttribute(UIConstants.AJAX_RATING_BEAN, ajaxRatingAction);
                request.getRequestDispatcher(UIConstants.AJAX_RATING_JSP).forward(request, response);

            } else if (command.equals("/getRating")) {

                String rating = request.getParameter("userRating");
                int iRating = new Integer(rating).intValue();

                AjaxRatingAction ajaxRatingAction = new AjaxRatingAction();
                ajaxRatingAction.setResourcePath(request.getParameter("resourcePath"));
                ajaxRatingAction.setUserRating(iRating);
                try {
                    ajaxRatingAction.execute(request);
                } catch (RegistryException e) {
                    e.printStackTrace();
                }

                request.getSession().setAttribute(UIConstants.AJAX_RATING_BEAN, ajaxRatingAction);
                request.getRequestDispatcher(UIConstants.AJAX_RATING_JSP).forward(request, response);

            } else if (command.equals("/getDescription")) {

                GetDescriptionAction getDescriptionAction = new GetDescriptionAction();
                getDescriptionAction.setResourcePath(request.getParameter("resourcePath"));

                try {
                    getDescriptionAction.execute(request);
                } catch (RegistryException e) {
                    setErrorMessage(request, e.getMessage());
                    e.printStackTrace();
                }

                request.getSession().setAttribute(UIConstants.AJAX_DESCRIPTION_STRING, getDescriptionAction.getDescription());
                request.getRequestDispatcher(UIConstants.AJAX_DESCRIPTION_JSP).forward(request, response);

            } else if (command.equals("/deleteResource")) {

                DeleteResourceAction deleteResourceAction = new DeleteResourceAction();
                deleteResourceAction.setPathToDelete(request.getParameter("resourcePath"));

                try {
                    deleteResourceAction.execute(request);
                } catch (Exception e) {
                    setErrorMessage(request, e.getMessage());
                }

                response.sendRedirect("/wso2registry/web" + path);

            } else if (command.equals("/getResourceMediaTypes")) {

                String mediaTypes = null;
                try {
                    mediaTypes  = MediaTypeMappingsReader.getMediaTypeMappings(request);
                } catch (RegistryException e) {
                    mediaTypes = "txt:text,wsdl:wsdl/xml,xsd:xsd/xml,iml:idea/proj,ipr:idea/projs";
                }

                response.getWriter().write(mediaTypes);

                return;
            }

        }
    }

    private void sendResourceContent(HttpServletRequest request, HttpServletResponse response, String path) {

        ResourceHandlerAction handlerAction = new ResourceHandlerAction();

        Resource resource = null;
        try {
            resource = handlerAction.getResource(request, path);
        } catch (RegistryException e) {
            setErrorMessage(request, e.getMessage());
            e.printStackTrace();
        }

        try {
            Object content = resource.getContent();
            if (content != null) {
                if (content instanceof byte[]) {
                    response.setContentType("application/download");
                    response.getOutputStream().write((byte[])content);
                    response.flushBuffer();
                } else {
                    response.setContentType("application/download");
                    response.getWriter().write(content.toString());
                }
            }

            response.getWriter().flush();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    /**
     * To find the name of the war , so getting that from the request context path
     *
     * @param req :  HttpServletRequest
     */
    public void initContextRoot(HttpServletRequest req) {
        if (contextRoot != null && contextRoot.trim().length() != 0) {
            return;
        }
        String contextPath = req.getContextPath();
//handling ROOT scenario, for servlets in the default (root) context, this method returns ""
        if (contextPath != null && contextPath.length() == 0) {
            contextPath = "/";
        }
        this.contextRoot = contextPath;
    }

    private String getParentPath(String resourcePath) {

        String parentPath;
        if (resourcePath.equals(RegistryConstants.ROOT_PATH)) {
            parentPath = null;
        } else {
            String formattedPath = resourcePath;
            if (resourcePath.endsWith(RegistryConstants.PATH_SEPARATOR)) {
                formattedPath = resourcePath.substring(
                        0, resourcePath.length() - RegistryConstants.PATH_SEPARATOR.length());
            }

            if (formattedPath.lastIndexOf(RegistryConstants.PATH_SEPARATOR) == 0) {
                parentPath = RegistryConstants.ROOT_PATH;
            } else {
                parentPath = formattedPath.substring(
                        0, formattedPath.lastIndexOf(RegistryConstants.PATH_SEPARATOR));
            }
        }

        return parentPath;
    }

    private void forwardToResources(HttpServletRequest request, HttpServletResponse response, String path) {

        try {
            ResourceDetailsAction details = new ResourceDetailsAction();
            details.setPath(path);
            details.execute(request);

            CollectionViewAction collection = null;
            if (details.isCollection()) {
                collection = new CollectionViewAction();
                collection.setPath(path);
                collection.execute(request);
            }

            request.getSession().setAttribute(UIConstants.RESOURCE_BEAN, details);
            request.getSession().setAttribute(UIConstants.COLLECTION_BEAN, collection);

        } catch (Exception e) {
            setErrorMessage(request, e.getMessage());
            e.printStackTrace();
        }

        try {
            request.getRequestDispatcher(UIConstants.RESOURCES_JSP)
                    .forward(request, response);
        } catch (ServletException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }

    }

    private void forwardToUserManagement(HttpServletRequest request, HttpServletResponse response) {

        UserManagementAction userManagementAction = new UserManagementAction();
        try {
            userManagementAction.execute(request);
        } catch (Exception e) {
            setErrorMessage(request, e.getMessage());
            e.printStackTrace();
        }

        request.getSession().setAttribute(UIConstants.USER_MANAGEMENT_BEAN, userManagementAction);

        try {
            request.getRequestDispatcher(UIConstants.USER_MANAGEMENT_JSP).forward(request, response);

        } catch (ServletException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    private void forwardToActivity(HttpServletRequest request, HttpServletResponse response) {

        RecentActivityAction recentActivityAction = new RecentActivityAction();
        recentActivityAction.setUserName(request.getParameter("userName"));
        recentActivityAction.setResourcePath(request.getParameter("path"));
        recentActivityAction.setFromDate(request.getParameter("fromDate"));
        recentActivityAction.setToDate(request.getParameter("toDate"));
        recentActivityAction.setFilter(request.getParameter("filter"));

        try {
            recentActivityAction.execute(request);
        } catch (Exception e) {
            setErrorMessage(request, e.getMessage());
            e.printStackTrace();
        }

        request.getSession().setAttribute(UIConstants.ACTIVITY_BEAN, recentActivityAction);

        try {
            request.getRequestDispatcher(UIConstants.ACTIVITY_JSP).forward(request, response);

        } catch (ServletException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    private void forwardToEmptyActivity(HttpServletRequest request, HttpServletResponse response) {

        RecentActivityAction recentActivity = new RecentActivityAction();
        recentActivity.setRequest(request);
        request.getSession().setAttribute(UIConstants.ACTIVITY_BEAN, recentActivity);

        try {
            request.getRequestDispatcher(UIConstants.ACTIVITY_JSP).forward(request, response);
        } catch (ServletException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    private void forwardToNewAdvancedSearch(HttpServletRequest request, HttpServletResponse response) {

        AdvancedSearchAction advancedSearchAction = new AdvancedSearchAction();
        advancedSearchAction.setRequest(request);
        request.getSession().setAttribute(UIConstants.ADVANCED_SEARCH_BEAN, advancedSearchAction);

        try {
            request.getRequestDispatcher(UIConstants.ADVANCED_SEARCH_JSP).forward(request, response);
        } catch (ServletException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    private void forwardToAdvancedSearch(HttpServletRequest request, HttpServletResponse response) {

        AdvancedSearchAction advancedSearchAction = new AdvancedSearchAction();
        advancedSearchAction.setResourcePath(request.getParameter("resourcePath"));
        advancedSearchAction.setAuthorName(request.getParameter("author"));
        advancedSearchAction.setUpdaterName(request.getParameter("updater"));
        advancedSearchAction.setTags(request.getParameter("tags"));
        advancedSearchAction.setCommentWords(request.getParameter("commentWords"));

        try {
            advancedSearchAction.execute(request);
        } catch (RegistryException e) {
            setErrorMessage(request, e.getMessage());
            e.printStackTrace();
        }

        request.getSession().setAttribute(UIConstants.ADVANCED_SEARCH_BEAN, advancedSearchAction);

        try {
            request.getRequestDispatcher(UIConstants.ADVANCED_SEARCH_JSP).forward(request, response);
        } catch (ServletException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    private void forwardToSearch(HttpServletRequest request, HttpServletResponse response) {

        SearchAction searchAction = new SearchAction();
        searchAction.setTag(request.getParameter("criteria"));

        try {
            searchAction.execute(request);
        } catch (Exception e) {
            setErrorMessage(request, e.getMessage());
            e.printStackTrace();
        }

        request.getSession().setAttribute(UIConstants.SEARCH_BEAN, searchAction);

        try {
            request.getRequestDispatcher(UIConstants.SEARCH_JSP).forward(request, response);
        } catch (ServletException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    private void forwardToUserPage(HttpServletRequest request, HttpServletResponse response, String userName) {

        String uri = request.getRequestURI();

        UserDetailsAction userDetailsAction = new UserDetailsAction();
        userDetailsAction.setUserName(userName);

        try {
            userDetailsAction.execute(request);
        } catch (Exception e) {
            setErrorMessage(request, e.getMessage());
            e.printStackTrace();
        }

        request.getSession().setAttribute(UIConstants.USER_BEAN, userDetailsAction);

        try {
            request.getRequestDispatcher(UIConstants.USER_JSP).forward(request, response);
        } catch (ServletException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    private void setErrorMessage(HttpServletRequest request, String message) {
        request.getSession().setAttribute(UIConstants.ERROR_MESSAGE, message);
    }
}
