/*
 * Copyright (c) 2006, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.web;

import org.wso2.registry.RegistryException;

import javax.servlet.http.HttpServletRequest;
import java.io.InputStream;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.IOException;

public class MediaTypeMappingsReader {

    public static final String MAPPINGS_FILE = "/WEB-INF/media-type-mappings.conf";

    public static String getMediaTypeMappings(HttpServletRequest request) throws RegistryException {

        InputStream input =
                request.getSession().getServletContext().getResourceAsStream(MAPPINGS_FILE);
        BufferedReader reader = new BufferedReader(new InputStreamReader(input));

        String mediaTypeString = null;
        try {
            while(reader.ready()) {
                String mediaTypeData = reader.readLine();
                if (mediaTypeString == null) {
                    mediaTypeString = mediaTypeData;
                } else {
                    mediaTypeString = mediaTypeString + "," + mediaTypeData;
                }
            }

        } catch (IOException e) {
            String msg = "Could not read the media type mappings file from the location: " + MAPPINGS_FILE;
            throw new RegistryException(msg);

        } finally {
            try {
                reader.close();
                input.close();
            } catch (IOException ignore) {
            }
        }

        return mediaTypeString;
    }
}
