/*
 * Copyright (c) 2006, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.web.actions;

import org.wso2.registry.*;
import org.wso2.registry.secure.AuthorizationFailedException;
import org.wso2.registry.web.actions.utils.ResourceData;

import javax.servlet.http.HttpServletRequest;
import java.util.List;
import java.util.ArrayList;

public class SearchAction extends AbstractRegistryAction {

    /**
     * Generate the collection view from the results of the tag search. This scenario is implemented
     * explicitly (without using a custom query) as it is the most common search.
     */
    private String tag;

    /**
     * Path of a query.
     */
    private String queryPath;

    private List resourceDataList = new ArrayList();
    private String status;
    private String errorMessage;

    public String execute(HttpServletRequest request) throws RegistryException {

        setRequest(request);
        
        // get the resource for the queryPath
        // get all its children and fill ResourceOverview objects
        // add ResourceOverview object to the resourceDataList list

        Registry registry = getRegistry();

        String[] childPaths;

        if (tag != null && tag.length() != 0) {
            TaggedResourcePath[] taggedPaths = registry.getResourcePathsWithTag(tag);
            childPaths = new String[taggedPaths.length];
            for (int i = 0; i < taggedPaths.length; i++) {
                childPaths[i] = taggedPaths[i].getResourcePath();
            }

        } else {

            Resource resource;
            try {
                resource = registry.get(queryPath);
            } catch (AuthorizationFailedException e) {
                status = "error";
                errorMessage = "User " + getUserName() +
                        " does not have permission to browse this collection.";
                return ERROR;
            }

            if (resource == null) {
                status = "error";
                errorMessage = "Requested collection at queryPath " + queryPath + " does not exists.";
                return ERROR;
            }

            childPaths = (String[]) resource.getContent();
        }

        for (int i = 0; i < childPaths.length; i++) {

            ResourceData resourceData = new ResourceData();
            resourceData.setResourcePath(childPaths[i]);

            if (childPaths[i] != null) {
                String[] parts = childPaths[i].split(RegistryConstants.PATH_SEPARATOR);
                resourceData.setName(parts[parts.length - 1]);
            }

            try {
                Resource child = registry.get(childPaths[i]);

                resourceData.setResourceType(
                        child.isDirectory()? ResourceData.COLLECTION : ResourceData.RESOURCE);
                resourceData.setAuthorUserName(child.getAuthorUserName());
                resourceData.setDescription(child.getDescription());
                resourceData.setAverageRating(registry.getAverageRating(child.getPath()));
                resourceData.setCreatedOn(child.getCreatedTime());

            } catch (AuthorizationFailedException e) {

                resourceData.setResourceType(ResourceData.UNKNOWN);
            }

            resourceDataList.add(resourceData);
        }

        status = "success";
        return SUCCESS;
    }

    public String getTag() {
        return tag;
    }

    public void setTag(String tag) {
        this.tag = tag;
    }

    public String getQueryPath() {
        return queryPath;
    }

    public void setQueryPath(String queryPath) {
        this.queryPath = queryPath;
    }

    public List getResourceDataList() {
        return resourceDataList;
    }

    public void setResourceDataList(List resourceDataList) {
        this.resourceDataList = resourceDataList;
    }

    public String getStatus() {
        return status;
    }

    public void setStatus(String status) {
        this.status = status;
    }

    public String getErrorMessage() {
        return errorMessage;
    }

    public void setErrorMessage(String errorMessage) {
        this.errorMessage = errorMessage;
    }

}
